%==========================================================================
%  Function  rk() 
%     implements the Runge-Kutta algorithm presented in the lecture notes 
%     at the end of chapter 2.8. It simulates one time step a (nonlinear)
%     ordinary differential equation. 
%     In detail, for a system represented by an ODE of the form 
%     dx/dt = my_ode(x,u ), the function rk() calculates for a given 
%     state x (scalar or vector) and a given control input u (vector or 
%     scalar) the resulting state after the time delta_t. The control 
%     input is assumed to be constant during the step delta_t.
% 
%  Synopsis:
%     x = rk(my_ode, delta_t, x, u)
% 
%  Input Parameter:
%     - my_ode: ODE of the form dx/dt = f(x,u) that has to be integrated
%               <function handle to function f where f is implemented such 
%               that inputs x, u and outputs dx/dt are column vectors
%     - x: state at beginning of time interval
%          <column vector>
%     - delta_t: length of considered time interval
%                <positive float>
%     - u: control input of ode; assumed to be constant during the time 
%          interval; <column vector>
% 
%  Output Parameter:
%     - state at the end of the time interval
%       <column vector>
%
%  Author: Joerg Fischer, Lukas Klar
%  Systems control and optimization laboratory, 
%  Department of Microsystems Engineering (IMTEK),
%  University of Freiburg
% 
% File Created: 28.04.2016
%==========================================================================
function [xout] = rk(my_ode, delta_t, x, u)
   k1 = my_ode(x, u);
   k2 = my_ode(x + delta_t / 2 * k1 , u);
   k3 = my_ode(x + delta_t / 2 * k2 , u);
   k4 = my_ode(x + delta_t * k3, u);
   xout = (x + delta_t * (k1 + 2 * k2 + 2* k3 + k4) / 6);
end